#!/usr/bin/env python
# -*- coding: utf-8; tab-width: 4; mode: python -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: t -*-
# vi: set ft=python sts=4 ts=4 sw=4 noet 

# This file is part of Tivion.

# Tivion
# Copyright (C) 2009-2012 Ángel Guzmán Maeso
# http://shakaran.net
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# Author: Ángel Guzmán Maeso
# 
# $Revision$

# Useful guide: http://docs.python.org/distutils/builtdist.html
# setup.py --help

"""
Required PyGTK:
You can download PyGTK 2.16 (lastest for now) from:
http://ftp.gnome.org/pub/GNOME/binaries/win32/pygtk/2.16/pygtk-2.16.0.win32-py2.6.exe

PyObject:
http://ftp.gnome.org/pub/GNOME/binaries/win32/pygobject/2.20/pygobject-2.20.0.win32-py2.6.exe


Commandline for Windows (from root folder of project):
python setup.py bdist_wininst --bitmap src/data/img/tivion.bmp --title "Tivion: A simple streaming player" --dist-dir dist/windows/

python setup.py bdist_wininst --bitmap src/data/img/tivion.bmp --title "Tivion: A simple streaming player" --dist-dir dist/windows/ --install-script=src/tivion_postinstall.py --user-access-control force --target-version 2.6 -p win32
python setup.py bdist_wininst --bitmap src/data/img/tivion.bmp --title "Tivion: A simple streaming player" --dist-dir dist/windows/ --install-script=src/tivion_postinstall.py --user-access-control force --target-version 2.6 -p win-amd64
python setup.py bdist_wininst --bitmap src/data/img/tivion.bmp --title "Tivion: A simple streaming player" --dist-dir dist/windows/ --install-script=src/tivion_postinstall.py --user-access-control force --target-version 2.6 -p win-ia64

For debug:
set DISTUTILS_DEBUG=True
"""
import logging
log = logging.getLogger('TivionSetup')

import platform

import distutils
import distutils.file_util
import distutils.sysconfig
if platform.system() == 'Windows':
    import distutils.command.bdist_msi
from distutils.command.build import show_compilers
from distutils.core import setup
from distutils.command.install_data import install_data
from distutils.command.install_lib import install_lib
from distutils.command.bdist_wininst import bdist_wininst
from distutils.sysconfig import get_python_lib


import sys

if sys.version_info[:3] < (3, 0, 0):
    log.error("Python 3.0 or higher is required, %d.%d.%d found" % sys.version_info[:3])
    sys.exit(1)

try:
    from gi.repository import Gtk
except:
    print ('You need install GTK to run Tivion.')
    sys.exit(1)

log.debug('Gtk {0}.{1}.{2}'.format(Gtk.get_major_version(), Gtk.get_minor_version(), Gtk.get_micro_version()))

if (Gtk.get_major_version(), Gtk.get_minor_version()) < (3, 6):
    log.error('You need GTK 3.6 or higher to run Tivion.')
    sys.exit(1)

class post_install(install_data):
    def run(self):
        print ('=== Post install script ===')
         
        install_cmd = self.get_finalized_command('install')
        print ('Install dir:', getattr(install_cmd, 'install_lib'))
        #self.install_dir = getattr(install_cmd, 'install_lib')

        install_data.run(self) # Call parent
        # Execute commands
        print


class post_install_lib(install_lib):
    def run(self):
        print ('=== Post install lib script ===')
        install_lib.run(self)
        print
        
        
class pre_bdist_wininst(bdist_wininst):
    
    #def get_inidata(self):
        #return bdist_wininst.get_inidata(bdist_wininst) + 'Hola'
        #return super(pre_bdist_wininst, self)
        #.get_inidata() + 'TODO'
        
    def run(self):
        #[TODO] Make a translation for the installer and fix some
        # variables name like author_email
        print ('=== Pre bdist_wininst script ===')
        #print self.get_inidata()
        #print dir(bdist_wininst)
        #print self.get_inidata()
        bdist_wininst.run(self)
        print
        #get_inidata
      
        
class post_bdist_wininst(bdist_wininst):

    def run(self):
        print ('=== Post bdist_wininst script ===')
        bdist_wininst.run(self)
        print

#import _winreg
#import win32api
#import win32security


# patch distutils if it can't cope with the "classifiers" or
# "download_url" keywords

import os
import sys

from sys import version
if version < '2.2.3':
    try:
        # add necessary to distutils (for backwards compatibility)
        from distutils.dist import DistributionMetadata
        DistributionMetadata.classifiers = None
        DistributionMetadata.download_url = None
        DistributionMetadata.platforms = None
    except:
        pass

# Get current Python version
python_version = platform.python_version_tuple()

# Setup the default install prefix
prefix = sys.prefix

# Check our python is version 2.6 or higher
if python_version[0] >= 2 and python_version[1] >= 6:
    ## Set file location prefix accordingly
    prefix = '/usr/local'

# Get the install prefix if one is specified from the command line
for arg in sys.argv:
    if arg.startswith('--prefix='):
        prefix = arg[9:]
        prefix = os.path.expandvars(prefix)
    """if arg.startswith('bdist_wininst'): # bdist_wise
        #[TODO]Put windows path here
    if arg.startswith('bdist_rpm'):
        #[TODO]Put fedora path here"""

import tivion

DEBUG = False

print ('=== Installer for', tivion.__version__, '===')
print ('Plataform:', distutils.util.get_platform())
lib_dir = get_python_lib(plat_specific = 1)
print ('Lib dir:', lib_dir)
if DEBUG:
    print (show_compilers())
print

      
"""
try:
    prg = get_special_folder_path("CSIDL_COMMON_PROGRAMS")
except OSError:
    try:
        prg = get_special_folder_path("CSIDL_PROGRAMS")
    except OSError, reason:
        # give up - cannot install shortcuts
        print "cannot install shortcuts: %s" % reason
        sys.exit()
        
print "PRG", prg
"""

files = ['src/data/*']
#ValueError: path 'src/' cannot end with '/'
if platform.system() == 'Windows':
    scripts_data = None #['src/tivion.py']
else: 
   scripts_data = ['src/tivion']

if platform.system() == 'Windows':
    author_data = unicode('Ángel Guzmán Maeso')
else:
    author_data = 'Ángel Guzmán Maeso'

if platform.system() == 'Windows':
    usr_share = os.path.join('usr', 'share')#\\usr\\share\\'
else:
    usr_share = '/usr/share/'

if platform.system() == 'Windows':
    sep = os.sep
    src_dir = 'src' + os.sep
    img_dir = 'data\\img\\'
    lang_dir = '\\usr\\share\\locale'
    tivion_install_path = os.environ['HOMEDRIVE'] + sep + os.environ['PROGRAMFILES'] + '\\' + cons.VERSION_NAME+ '\\'
    #loptions = {'bdist_wininst': {'install_script': src_dir + 'pygtk_postinstall.py'}}
    #loptions = [{"bdist_wininst": {"install_script": "pygtk_postinstall.py"}}]
    #scripts = [src_dir + 'postinstall.py'],
else:
    sep = os.sep
    src_dir = 'src/'
    img_dir = 'data/img/'
    lang_dir = '/usr/share/locale'
    tivion_install_path = '/usr/share/tivion/'
    #loptions = []
    #scripts = []

"""
path = os.path.join(tivion_install_path, "Tivion.lnk")
arguments = os.path.join(lib_dir, "Tivion\\layoutEditor.py")
create_shortcut(pythonw, "Layout Editor", path, tivion_install_path + 'tivion.py')
file_created(path)
"""
"""
import win32com.client

shell = win32com.client.Dispatch("WScript.Shell")
shortcut = shell.CreateShortCut("t:\\test.lnk")
shortcut.Targetpath = "t:\\ftemp"
shortcut.save()
"""

data_files_installed = [
      
            ## Installing the executable
            (tivion_install_path, [src_dir + 'tivion.py']),
            
            ## Installing the channels, constants, utils and countries module
            (tivion_install_path, [src_dir + 'channels.py']),
            (tivion_install_path, [src_dir + 'constants.py']),
            (tivion_install_path, [src_dir + 'countries.py']),
            (tivion_install_path, [src_dir + 'utils.py']),
            
            ## Installing the data files (logo and images)
            (tivion_install_path + img_dir, [src_dir + img_dir + 'tivion.png']),
            (tivion_install_path + img_dir, [src_dir + img_dir + 'tivion-icon.png']),
            (tivion_install_path + img_dir, [src_dir + img_dir + 'tivion-icon.svg']),
            (tivion_install_path + img_dir, [src_dir + img_dir + 'tivion-launcher.png']),
            (tivion_install_path + img_dir, [src_dir + img_dir + 'tivion-frame.png']),
            (tivion_install_path + img_dir, [src_dir + img_dir + 'all.png']),
            (tivion_install_path + img_dir, [src_dir + img_dir + 'media-playback-start.png']),
            (tivion_install_path + img_dir, [src_dir + img_dir + 'pause_bg.png']),
            (tivion_install_path + img_dir, [src_dir + img_dir + 'play_bg.png']),
            (tivion_install_path + img_dir, [src_dir + img_dir + 'radio.png']),
            (tivion_install_path + img_dir, [src_dir + img_dir + 'sopcast.png']),
            (tivion_install_path + img_dir, [src_dir + img_dir + 'tv.png']),
            (tivion_install_path + img_dir, [src_dir + img_dir + 'camera-photo.png']),
            (tivion_install_path + img_dir, [src_dir + img_dir + 'audio-volume-muted.png']),
            (tivion_install_path + img_dir, [src_dir + img_dir + 'audio-volume-high.png']),
            (tivion_install_path + img_dir, [src_dir + img_dir + 'audio-volume-low.png']),
            (tivion_install_path + img_dir, [src_dir + img_dir + 'audio-volume-medium.png']),
            (tivion_install_path + img_dir, [src_dir + img_dir + 'lpi-help.png']),
            (tivion_install_path + img_dir, [src_dir + img_dir + 'lpi-translate.png']),
            (tivion_install_path + img_dir, [src_dir + img_dir + 'lpi-bug.png']),
            (tivion_install_path + img_dir, [src_dir + img_dir + 'tivion.bmp']),
            (tivion_install_path + img_dir, [src_dir + img_dir + 'tivion.xcf']),

            ## Installing the locale files
            
            # Catala
            (lang_dir + sep + 'ca' + sep + 'LC_MESSAGES' + sep, 
            [src_dir + 'lang' + sep + 'ca' + sep + 'LC_MESSAGES' + sep + 'tivion.po']),
            (lang_dir + sep + 'ca' + sep + 'LC_MESSAGES' + sep, 
            [src_dir + 'lang' + sep + 'ca' + sep + 'LC_MESSAGES' + sep + 'tivion.mo'])
            
            ]
            
if platform.system() == 'Windows':
    data_files_installed.append((tivion_install_path, [src_dir + 'bin//mplayer.exe']))
    data_files_installed.append((tivion_install_path, [src_dir + 'Tivion.lnk']))
    data_files_installed.append((os.environ['HOMEDRIVE'] + os.sep + os.environ['HOMEPATH'] + os.sep + 'Desktop', [src_dir + 'Tivion.lnk']))
    data_files_installed.append((os.environ['HOMEDRIVE'] + os.sep + os.environ['HOMEPATH'] + os.sep + 'Escritorio', [src_dir + 'Tivion.lnk']))

setup(
      name             = 'tivion',
      version          = tivion.__version__,
      description      = 'A simple player to online TV streaming',
      long_description = tivion.__long__description__,
      author           = author_data,
      author_email     = 'shakaran@gmail.com',
      maintainer       = tivion.__maintainer__,
      maintainer_email = tivion.__maintainer_email__,
      url              = tivion.__url__,
      download_url     = 'http://www.shakaran.net/blog/tivion',
      packages         = [''],
      package_data     = {'' : ''}, # files
      #scripts = scripts,
      license          = tivion.__license_fullname__, #, 
      platforms        = 'Python 3.0 and later',
      data_files       = data_files_installed,
      py_modules       = [],
      scripts          = [src_dir + 'tivion_postinstall.py'],
      cmdclass         = {'install_data': post_install, 
                          'install_lib': post_install_lib, 
                          'bdist_wininst': pre_bdist_wininst,
                          'install_script': post_bdist_wininst}#,
                          #options = loptions
)


""" 
[TODO]
 # English
('src/lang/', ['/usr/share/locale/en/LC_MESSAGES/tivion.po']),
('src/lang/', ['/usr/share/locale/en/LC_MESSAGES/tivion.mo']),

# English (United Kingdom)
('src/lang/', ['/usr/share/locale/en_GB/LC_MESSAGES/tivion.po']),
('src/lang/', ['/usr/share/locale/en_GB/LC_MESSAGES/tivion.mo']),

# Spanish
('src/lang/', ['/usr/share/locale/es/LC_MESSAGES/tivion.po']),
('src/lang/', ['/usr/share/locale/es/LC_MESSAGES/tivion.mo']),

# French
('src/lang/', ['/usr/share/locale/fr/LC_MESSAGES/tivion.po']),
('src/lang/', ['/usr/share/locale/fr/LC_MESSAGES/tivion.mo']),
 
# Italian
('src/lang/', ['/usr/share/locale/it/LC_MESSAGES/tivion.po']),
('src/lang/', ['/usr/share/locale/it/LC_MESSAGES/tivion.mo']),

# Portuguese
('src/lang/', ['/usr/share/locale/pt/LC_MESSAGES/tivion.po']),
('src/lang/', ['/usr/share/locale/pt/LC_MESSAGES/tivion.mo']),

# Russian
('src/lang/', ['/usr/share/locale/ru/LC_MESSAGES/tivion.po']),
('src/lang/', ['/usr/share/locale/ru/LC_MESSAGES/tivion.mo']),

# Turkish
('src/lang/', ['/usr/share/locale/tr/LC_MESSAGES/tivion.po']),
('src/lang/', ['/usr/share/locale/tr/LC_MESSAGES/tivion.mo']),

# Italian
('src/lang/', ['/usr/share/locale/it/LC_MESSAGES/tivion.po']),
('src/lang/', ['/usr/share/locale/it/LC_MESSAGES/tivion.mo']),

# Portuguese
('src/lang/', ['/usr/share/locale/pt/LC_MESSAGES/tivion.po']),
('src/lang/', ['/usr/share/locale/pt/LC_MESSAGES/tivion.mo']),

# Russian
('/usr/share/locale/ru/LC_MESSAGES/', ['src/lang/ru/LC_MESSAGES/tivion.po']),
('/usr/share/locale/ru/LC_MESSAGES/', ['src/lang/ru/LC_MESSAGES/tivion.mo']),

# Afrikaans
('/usr/share/locale/af/LC_MESSAGES/', ['src/lang/af/LC_MESSAGES/tivion.po']),
('/usr/share/locale/af/LC_MESSAGES/', ['src/lang/af/LC_MESSAGES/tivion.mo']),

# Aragonese
('/usr/share/locale/an/LC_MESSAGES/', ['src/lang/an/LC_MESSAGES/tivion.po']),
('/usr/share/locale/an/LC_MESSAGES/', ['src/lang/an/LC_MESSAGES/tivion.mo']),

# Arabic
('/usr/share/locale/ar/LC_MESSAGES/', ['src/lang/ar/LC_MESSAGES/tivion.po']),
('/usr/share/locale/ar/LC_MESSAGES/', ['src/lang/ar/LC_MESSAGES/tivion.mo']),

# Bosnian
('/usr/share/locale/bs/LC_MESSAGES/', ['src/lang/bs/LC_MESSAGES/tivion.po']),
('/usr/share/locale/bs/LC_MESSAGES/', ['src/lang/bs/LC_MESSAGES/tivion.mo']),

# Greek
('/usr/share/locale/el/LC_MESSAGES/', ['src/lang/el/LC_MESSAGES/tivion.po']),
('/usr/share/locale/el/LC_MESSAGES/', ['src/lang/el/LC_MESSAGES/tivion.mo']),

# English (Australia)
('/usr/share/locale/en_AU/LC_MESSAGES/', ['src/lang/en_AU/LC_MESSAGES/tivion.po']),
('/usr/share/locale/en_AU/LC_MESSAGES/', ['src/lang/en_AU/LC_MESSAGES/tivion.mo']),

# Basque
('/usr/share/locale/eu/LC_MESSAGES/', ['src/lang/eu/LC_MESSAGES/tivion.po']),
('/usr/share/locale/eu/LC_MESSAGES/', ['src/lang/eu/LC_MESSAGES/tivion.mo']),

# Albanian
('/usr/share/locale/sq/LC_MESSAGES/', ['src/lang/sq/LC_MESSAGES/tivion.po']),
('/usr/share/locale/sq/LC_MESSAGES/', ['src/lang/sq/LC_MESSAGES/tivion.mo']),

# Klingon
('/usr/share/locale/tlh/LC_MESSAGES/', ['src/lang/tlh/LC_MESSAGES/tivion.po']),
('/usr/share/locale/tlh/LC_MESSAGES/', ['src/lang/tlh/LC_MESSAGES/tivion.mo']),

# Chinese (Traditional)
('/usr/share/locale/zh_TW/LC_MESSAGES/', ['src/lang/zh_TW/LC_MESSAGES/tivion.po']),
('/usr/share/locale/zh_TW/LC_MESSAGES/', ['src/lang/tlh/LC_MESSAGES/tivion.mo']),

# German
('/usr/share/locale/de/LC_MESSAGES/', ['src/lang/de/LC_MESSAGES/tivion.po']),
('/usr/share/locale/de/LC_MESSAGES/', ['src/lang/de/LC_MESSAGES/tivion.mo']),

# Galician
('/usr/share/locale/gl/LC_MESSAGES/', ['src/lang/gl/LC_MESSAGES/tivion.po']),
('/usr/share/locale/gl/LC_MESSAGES/', ['src/lang/gl/LC_MESSAGES/tivion.mo']),

# Croatian
('/usr/share/locale/hr/LC_MESSAGES/', ['src/lang/hr/LC_MESSAGES/tivion.po']),
('/usr/share/locale/hr/LC_MESSAGES/', ['src/lang/hr/LC_MESSAGES/tivion.mo']),

# Indonesian
('/usr/share/locale/id/LC_MESSAGES/', ['src/lang/id/LC_MESSAGES/tivion.po']),
('/usr/share/locale/id/LC_MESSAGES/', ['src/lang/id/LC_MESSAGES/tivion.mo']),          

## Installing menufile
('src/', ['/usr/share/applications/tivion.desktop']),

## Readme and documentation files
#('.', ['/usr/share/doc/tivion/README']),

],

"""

  #"""[TODO]data_files=[('/usr/share/tivion/', ['bm/b1.gif', 'bm/b2.gif']),
  #            ('config', ['cfg/data.cfg']),
  #            ('/etc/init.d', ['init-script'])],"""
  #platforms = ['Linux'],
  #scripts = ["runner"],
  #cmdclass={'install': InstallAndUpdate},
  #
  #windows = ["tivion.py"],
  #options = {"py2exe": {"bundle_files": 1}},
  #
      #scripts = scripts_data,
  #"""classifiers = [
  #    'Development Status :: 1 - Alpha',
  #    'Environment :: Console',
  #    'Environment :: Web Environment',
  #    'Intended Audience :: End Users/Desktop',
  #    'Intended Audience :: Developers',
  #    'Intended Audience :: System Administrators',
  #    'License :: OSI Approved :: GNU Public Licence Version 3',
  #    'Operating System :: Linux :: Ubuntu 9.04',
  #    'Operating System :: Linux :: Ubuntu 9.10',
  #    'Operating System :: Linux :: Ubuntu 10.04',
  #     #'Operating System :: Microsoft :: Windows', # Not yet
  #     #'Operating System :: MacOS :: MacOS X', # Not yet
  #    'Programming Language :: Python',
  #    'Topic :: Internet :: Media',
  #    'Topic :: Internet :: Sound & Video',
  #    'Topic :: Multimedia :: Graphics :: Viewers',
  #    ],"""
