#!/usr/bin/env python
# -*- coding: utf-8 -*-

## Tivion
## Copyright (C) 2009-2010 Ángel Guzmán Maeso
## http://shakaran.net
##
## This program is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import with_statement
import sys
import os
import constants as cons
import ConfigParser
import shutil
import signal
from xml.dom.minidom import parseString;
import urllib
import re
import platform
import locale
import gettext
import gtk
from threading import Thread
import gobject
import time

cur_locale = locale.setlocale(locale.LC_ALL, None)
gettext.bindtextdomain('tivion', cons.LOCALE_DIR)
gettext.textdomain('tivion')
_ = gettext.gettext

class ChannelUpdater(Thread):

    def __init__(self):
        Thread.__init__(self)

        self.quit = False

    def get_channels(self):
        print _('Init the automatic channel updater...')
        channels_data = get_channel_data()
        if channels_data != None:
            new_channel_version = re.findall("CHANNEL_VERSION \= \'(.+?)\'", channels_data)

            from channels import CHANNEL_VERSION, CHANNEL
            from datetime import date
            if cons.DEBUG:
                print _('OLD channel version (timestamp):'), CHANNEL_VERSION, _('with'), len(CHANNEL), 'channels'
                print _('NEW channel version (timestamp):'), new_channel_version[0]

            print _('OLD channel version:'), time.strftime(_("%A %d, %B %G"), date.fromtimestamp(float(CHANNEL_VERSION)).timetuple()), _('with'), len(CHANNEL), 'channels'
            print _('NEW channel version:'), time.strftime(_("%A %d, %B %G"), date.fromtimestamp(float(new_channel_version[0])).timetuple())

            if new_channel_version[0] > CHANNEL_VERSION:
                print _('Found new version, updating the channel list')

                update_dialog = gtk.MessageDialog(None,
                                        gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_QUESTION,
                                        0, _('New version channel found:\n') +
                                           _('OLD channel version: ') + str(time.strftime(_("%A %d, %B %G"), date.fromtimestamp(float(CHANNEL_VERSION)).timetuple())) + _(' with ') + str(len(CHANNEL)) + _(' channels') + '\n' +
                                           _('NEW channel version: ') + str(time.strftime(_("%A %d, %B %G"), date.fromtimestamp(float(new_channel_version[0])).timetuple())) + '\n' +
                                           _('Do you want update the channel list?'))
                update_dialog.set_modal(True)
                update_dialog.set_title(_('¡New version channel list found!'))
                update_dialog.set_position(gtk.WIN_POS_CENTER)
                update_dialog.add_button(gtk.STOCK_YES, gtk.RESPONSE_OK)
                update_dialog.add_button(gtk.STOCK_NO, gtk.RESPONSE_CANCEL)
                update_dialog.set_default_response(gtk.RESPONSE_CANCEL)

                response = update_dialog.run()

                if response == gtk.RESPONSE_OK:
                    write_temp_channel(channels_data)

                    print _('Copyng new channels')
                    os.system('gksudo "mkdir -p /usr/share/tivion" && gksudo "cp -rf /tmp/channels.py /usr/share/tivion/channels.py"')
                    print _('Channels update completed.')

                update_dialog.destroy()

            else:
                print _('You have the lastest channel version. No new channels found.')
        self.quit = True

    def run(self):
        try:
            if cons.DEBUG:
                print _('Thread: getting and updating channels...')
            self.get_channels()
        except KeyboardInterrupt:
            self.quit = True
        except OSError, e:
            print _('Error: OSError with channels updater thread: '), e
        except IOError, e:
            print _('Error: IOError with channels updater thread: '), e
        except Exception, e:
            print _('Error: Error with channels updater thread: '), e

def get_default_lang():
    # windows hack for locale setting
    if platform.system() == 'Windows':
        lang = os.getenv('LANG')
        if lang is None:
            default_lang, default_enc = locale.getdefaultlocale()
            if default_lang:
                lang = default_lang
            if lang:
                os.environ['LANG'] = lang
    return os.environ['LANG']

def get_mplayer_bin():
    if platform.system() == 'Windows':
        return os.environ['PROGRAMFILES'] + '/' + cons.VERSION_NAME + '/mplayer.exe'
    else:
        return '/usr/bin/mplayer'

def check_player():
    from os.path import exists as e
    return e(get_mplayer_bin()) # and e("/usr/bin/serpentine")

def get_home():
    if platform.system() == 'Windows':
        return os.environ['HOMEPATH']
    else:
        return os.environ['HOME']

def check_sopcast_player():
    from os.path import exists as e
    return e(os.path.join(cons.LIB_PATH, 'sp-sc-auth')) or e(os.path.join(cons.DEV_LIB_PATH, 'sp-sc-auth'))

def user_agent(): # [TODO] Make a real user_agent based on user distro
    return 'Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.6) Gecko/2009020911 Ubuntu/8.10 (intrepid) Firefox/3.0.6'

def check_dir(path):
    if not os.path.exists(path):
        try:
            os.mkdir(path)
        except OSError:
            print _('Error, could not create dir \'%s\':' % path), e

    if not os.path.isdir(path):
        shutil.copy2(path, path + '_backup')
        os.remove(path)
        try:
            os.mkdir(path)
        except OSError:
            print _('Error, could not create dir \'%s\':' % path), e

def pterminate(process):
    if sys.version_info >= (2, 6):
        if platform.system() != 'Windows':
            process.terminate()
        else:
            os.kill(process.pid, signal.SIGTERM)
    else:
        os.kill(process.pid, signal.SIGTERM)

def make_menu(items):
    menu = gtk.Menu()
    for item in items:
        menu.append(item)
    return menu

def get_channel_url():
    try:
        channels_url = urllib.urlopen('http://bazaar.launchpad.net/~shakaran/tivion/tivion/annotate/head:/src/channels.py').read()
    except IOError, e:
        print _('Error: could not connect to repository url. Check your connection or the repository URL.')
        return None
    channels_url = re.findall("<a href=\"(.+?)\">download file</a>", channels_url)
    if cons.DEBUG:
        # http://bazaar.launchpad.net/~shakaran/tivion/tivion/download/head:/channels.py-20090902173635-8zjl0u0xzhky5rv1-2/channels.py
        print _('Channels URL: ') + 'http://bazaar.launchpad.net' + channels_url[0]
    if channels_url != None:
        return 'http://bazaar.launchpad.net' + channels_url[0]
    else:
        print _('Warning: could not get the channel url list repository. Check your connection or the repository URL.')
        return None

def get_channel_data():
    url = get_channel_url()
    if url != None:
        return urllib.urlopen(url).read()
    else:
        return None

def write_temp_channel(data):
    f = open('/tmp/channels.py', 'w')
    f.write(data)
    f.close()

def ASX_parser(url):
    f = urllib.urlopen(url)
    try:
        data = parseString(f.read())
    except Exception, e:
        print >> sys.stderr, _('Invalid XML:'), e
        print _('Recieved data:')
        print f.read()
        data = None

    """
    <asx version="3.0">
      <title>100 XR - #1 Rock Web Station!!!</title>

      <entry>
        <title>100 XR - #1 Rock Web Station!!!</title>
        <ref href="http://208.53.158.48:8250" />
        <author>100 XR</author>
        <copyright>100xr.com</copyright>
      </entry>
    </asx>
    """
    if data:
        ref = data.getElementsByTagName('ref')
        if not ref:
            ref = data.getElementsByTagName('REF')
        if not ref:
            print _('Error: could not parse the ASX url \'%s\'' % url)
            return None

        links = [ r.getAttribute('HREF')
                  for r in ref
                  if r.hasAttribute('HREF') ]
        if not links:
            links = [ r.getAttribute('href')
                      for r in ref
                      if r.hasAttribute('href') ]
        if not links:
            print _('Error: could not parse the ASX link for \'%s\'' % url)
            return None

        if cons.DEBUG:
            for link in links:
                print _('Link:'), link

        return links[0]
    else:
        return None

class ButtonUnLabeled(gtk.Button):
    __gtype_name__ = 'ButtonUnLabeled'

    def __init__(self, stock = gtk.STOCK_CLOSE):
        super(ButtonUnLabeled, self).__init__(stock = stock)
        self.set_use_stock(True)
        self.get_children()[0].get_children()[0].get_children()[1].destroy()

class entrySearch(gtk.Entry):

    __gtype_name__ = 'entrySearch'
    app = None

    def __init__(self, app = None):
        gtk.Entry.__init__(self)
        self.app = app

        try: # Only PyGTK 2.16 or higher
          self.set_property('primary-icon-stock', gtk.STOCK_FIND)
          self.set_property('secondary-icon-stock', gtk.STOCK_CLEAR)
          self.connect('icon-press', self.on_icon_press)
        except:
            pass
        self.connect('key-press-event', self.on_key_press_event)
        self.popup_menu = self.__do_popup_menu()

    def __do_popup_menu(self):
        """
        Do the tv, radio and sopcast menu
        """

        # do menu
        menu = gtk.Menu()

        # items
        try:
            self.img_all     = gtk.gdk.pixbuf_new_from_file(os.path.join(cons.IMG_PATH, 'all.png'))
            self.img_radio   = gtk.gdk.pixbuf_new_from_file(os.path.join(cons.IMG_PATH, 'radio.png'))
            self.img_tv      = gtk.gdk.pixbuf_new_from_file(os.path.join(cons.IMG_PATH, 'tv.png'))
            self.img_sopcast = gtk.gdk.pixbuf_new_from_file(os.path.join(cons.IMG_PATH, 'sopcast.png'))
        except: # Give a try with development path
            self.img_all     = gtk.gdk.pixbuf_new_from_file(os.path.join(cons.DEV_IMG_PATH, 'all.png'))
            self.img_radio   = gtk.gdk.pixbuf_new_from_file(os.path.join(cons.DEV_IMG_PATH, 'radio.png'))
            self.img_tv      = gtk.gdk.pixbuf_new_from_file(os.path.join(cons.DEV_IMG_PATH, 'tv.png'))
            self.img_sopcast = gtk.gdk.pixbuf_new_from_file(os.path.join(cons.DEV_IMG_PATH, 'sopcast.png'))

        # append and connect to callback
        # [TODO] Put a gtk.Label on the popup menu with "Filter channel by:"
        menus = [ # gtk.Label('Filter channel by:'), # [TODO] GtkWarning: gtk_menu_shell_insert: assertion `GTK_IS_MENU_ITEM (child)' failed
                 ImageMenuItem(_('All'), self.img_all),
                 ImageMenuItem(_('TV'), self.img_tv),
                 ImageMenuItem(_('Radio'), self.img_radio),
                 ImageMenuItem(_('Sopcast'), self.img_sopcast)]
        for item in menus:
            menu.append(item)
            if type(item) == ImageMenuItem:
                item.connect('activate', self.on_menu_item_activated)

        menu.show_all()

        return menu

    def on_menu_item_activated(self, *args):
        """
        When user select from pop up
        menu we reset the treeview.
        """

        ## Use app.insert_channels
        # for reset model from here

        # this is a string 'TV', 'Radio', etc..
        index = args[0].get_label()
        if cons.DEBUG:
            print _('Item %s was pressed.' % index)

        if index:
            if index == 'TV':
                self.app.insert_channels(cons.TV)
            elif index == 'Radio':
                self.app.insert_channels(cons.RADIO)
            elif index == 'Sopcast':
                self.app.insert_channels(cons.SOPCAST)
            elif index == 'All':
                self.app.insert_channels(None)

    def clear(self):
        self.set_text('')
        self.grab_focus()

    def on_key_press_event(self, w, event):
        if event.type == gtk.gdk.KEY_PRESS:
            if gtk.gdk.keyval_name(event.keyval) == 'Return':
                self.app.model.clear()
                if self.get_text():
                    self.on_search(self.get_text())

    def on_icon_press(self, w, pos, event):
        if pos == gtk.ENTRY_ICON_PRIMARY:
            # The purpose of the primary icon should be to act as a filter,
            # to replace the combobox. When the user press it, a drop down
            # menu pops up with all, tv, radio and sopcast.

            # get the position of the search box, so that we can place the popup window in the appropriate location
            widget_pos = self.get_allocation()
            win_x, win_y = self.get_parent_window().get_root_origin()
            position = win_x + widget_pos.x + 2, win_y + widget_pos.y + widget_pos.height + self.size_request()[1]

            def positionFunc(menu, position):
                return position[0] , position[1], False

            #print self.popup_menu.get_children()#add_child(gtk.Label)
            #self.popup_menu.prepend(gtk.Label('hhola'))
            #print dir(self.popup_menu)

            self.popup_menu.popup(None, None, positionFunc, event.button, event.get_time(), position)

        elif pos == gtk.ENTRY_ICON_SECONDARY:
            if event.button == 1 and event.type == gtk.gdk.BUTTON_PRESS:
                return self.clear()

    def on_search(self, text):
        if cons.DEBUG:
            print 'Searching related channels to:', text
        self.app.insert_channels_by_text(None, text)


class ImageMenuItem(gtk.ImageMenuItem):

    __gtype_name__ = 'ImageMenuItem'

    def __init__(self, name = None, stock = None, callback = None, accel = None, accel_group = None, short_label = None):
        if platform.system() == 'Windows':
            self.__gobject_init__() # This is deprecated, but with Windows we only have pygtk 2.12
            gtk.ImageMenuItem.__init__(gtk.ImageMenuItem(name))
        else:
            try:
                super(ImageMenuItem, self).__init__(name)
            except: # Fallback mode
                self.__gobject_init__() # This is deprecated, but with Windows we only have pygtk 2.12
                gtk.ImageMenuItem.__init__(gtk.ImageMenuItem(name))

        if platform.system() == 'Windows':
            if name:
                self.set_property('use-underline', True)
                self.set_property('label', name)

        if stock:
            if type(stock) == gtk.gdk.Pixbuf:
                img = gtk.Image()
                img.set_from_pixbuf(stock)
                self.set_image(img)
            else:
                if stock in ('lpi-help', 'lpi-translate', 'lpi-bug'):
                    img = gtk.Image()
                    img.set_from_icon_name(stock, gtk.ICON_SIZE_MENU)
                    self.set_image(img)
                else:
                    self.set_image(gtk.image_new_from_stock(stock, gtk.ICON_SIZE_MENU))

        if accel:
            key, mod = gtk.accelerator_parse(accel)
            if accel_group:
                self.add_accelerator('activate', accel_group, key, mod, gtk.ACCEL_VISIBLE)

        if callback:
            self.connect('activate', callback)

class Config():
    cfg = None
    path_cfg_dir = None
    path_cfg_file = None

    def __init__(self):
        self.cfg = ConfigParser.ConfigParser()
        self.path_cfg_dir =  get_home() + '/.tivion'
        self.path_cfg_file =  get_home() + '/.tivion/' + cons.CONFIG_FILE
        self.__check_cfg_dir()
        self.__check_cfg_file()
        self.__read_cfg_file()
        self.__check_sections(['OPTIONS'])
        self.__check_options('OPTIONS', [('Lang', cur_locale), # [FIXME] Don't use cur_locale
                                         ('Screenshot_folder',  get_home() + '/.tivion/'+ _('captures')),
                                         ('Cache_size', cons.DEFAULT_CACHE_SIZE),
                                         ('Random_channel', '0'),
                                        ])

    def __check_cfg_dir(self):
        check_dir(self.path_cfg_dir)

    def __check_cfg_file(self):
        if not os.path.exists(self.path_cfg_file):
            try:
                open(self.path_cfg_file, 'w')
            except IOError:
                print _('Error, could not create config file \'%s\':' % self.path_cfg_file), e

        if not os.path.isfile(self.path_cfg_file):
            shutil.copy2(self.path_cfg_dir, self.path_cfg_dir + '_backup')
            os.remove(self.path_cfg_file)
            try:
                open(self.path_cfg_file, 'w')
            except IOError:
                print _('Error, could not create config file \'%s\':' % self.path_cfg_file), e

    def __read_cfg_file(self):
        try:
            self.cfg.readfp(file(self.path_cfg_file))
        except Exception, e:
            print _('Error, could not read config file \'%s\':' % self.path_cfg_file), e

    def __write_cfg_file(self):
        try:
            if sys.version_info < (2, 5):
                cfg_file = open(self.path_cfg_file, 'wb')
                self.cfg.write(cfg_file)
            else:
                with open(self.path_cfg_file, 'wb') as cfg_file:
                    self.cfg.write(cfg_file)
        except Exception, e:
            print e

    def write(self):
        self.__write_cfg_file()

    def __check_sections(self, sections):
        for s in sections:
            if not self.cfg.has_section(s):
                print _('Warning: section \'%s\' does not exists in conf file, created automatically.' % s)
                self.cfg.add_section(s)
                self.__write_cfg_file()

    def __check_options(self, section, options):
        for (op, default_value) in options:
            if not self.cfg.has_option(section, op):
                print _('Warning: section \'%s\' does not have the option ' %section) + _('\'%s\' in conf file, created automatically with default value' %op) +(' \'%s\'.' %default_value)
                self.cfg.set(section, op, default_value)
                self.__write_cfg_file()
